@@ -11,15 +11,15 @@
 
 import threading, sys
 
+
 class REThread(threading.Thread):
     '''Thread with return values and exception propagation.'''
 
     def __init__(self, group=None, target=None, name=None, args=(), kwargs={},
-            verbose=None):
+                 verbose=None):
         '''Initialize Thread, identical to threading.Thread.__init__().'''
 
-        threading.Thread.__init__(self, group, target, name, args, kwargs,
-            verbose)
+        threading.Thread.__init__(self, group, target, name, args, kwargs, verbose)
         self.__target = target
         self.__args = args
         self.__kwargs = kwargs
@@ -57,6 +57,10 @@ class REThread(threading.Thread):
         Do nothing if no exception was caught.
         '''
         if self._exception:
-            raise self._exception[0], self._exception[1], self._exception[2]
+            # there is no syntax which both Python 2 and 3 parse, so we need a
+            # hack using exec() here
             # Python 3:
-            #raise self._exception[0](self._exception[1]).with_traceback(self._exception[2])
+            if sys.version > '3':
+                raise self._exception[1].with_traceback(self._exception[2])
+            else:
+                exec('raise self._exception[0], self._exception[1], self._exception[2]')
