@@ -1,6 +1,6 @@
 '''Convenience functions for use in package hooks.'''
 
-# Copyright (C) 2008 - 2011 Canonical Ltd.
+# Copyright (C) 2008 - 2012 Canonical Ltd.
 # Authors:
 #   Matt Zimmerman <mdz@canonical.com>
 #   Brian Murray <brian@ubuntu.com>
@@ -39,6 +39,7 @@ except AttributeError:
     import string
     _path_key_trans = string.maketrans('#/-_+ ', '....._')
 
+
 def path_to_key(path):
     '''Generate a valid report key name from a file path.
 
@@ -52,7 +53,8 @@ def path_to_key(path):
             path = path.encode('UTF-8')
     return path.translate(_path_key_trans)
 
-def attach_file_if_exists(report, path, key=None, overwrite=True):
+
+def attach_file_if_exists(report, path, key=None, overwrite=True, force_unicode=False):
     '''Attach file contents if file exists.
 
     If key is not specified, the key name will be derived from the file
@@ -60,26 +62,40 @@ def attach_file_if_exists(report, path, key=None, overwrite=True):
 
     If overwrite is True, an existing key will be updated. If it is False, a
     new key with '_' appended will be added instead.
+
+    If the contents is valid UTF-8, or force_unicode is True, then the value
+    will a string, otherwise it will be bytes.
     '''
     if not key:
         key = path_to_key(path)
 
     if os.path.exists(path):
-        attach_file(report, path, key, overwrite)
+        attach_file(report, path, key, overwrite, force_unicode)
 
-def read_file(path):
+
+def read_file(path, force_unicode=False):
     '''Return the contents of the specified path.
 
+    If the contents is valid UTF-8, or force_unicode is True, then the value
+    will a string, otherwise it will be bytes.
+
     Upon error, this will deliver a text representation of the error,
     instead of failing.
     '''
     try:
         with open(path, 'rb') as f:
-            return f.read().strip()
+            contents = f.read().strip()
+        if force_unicode:
+            return contents.decode('UTF-8', errors='replace')
+        try:
+            return contents.decode('UTF-8')
+        except UnicodeDecodeError:
+            return contents
     except Exception as e:
         return 'Error: ' + str(e)
 
-def attach_file(report, path, key=None, overwrite=True):
+
+def attach_file(report, path, key=None, overwrite=True, force_unicode=False):
     '''Attach a file to the report.
 
     If key is not specified, the key name will be derived from the file
@@ -87,6 +103,9 @@ def attach_file(report, path, key=None, overwrite=True):
 
     If overwrite is True, an existing key will be updated. If it is False, a
     new key with '_' appended will be added instead.
+
+    If the contents is valid UTF-8, or force_unicode is True, then the value
+    will a string, otherwise it will be bytes.
     '''
     if not key:
         key = path_to_key(path)
@@ -95,7 +114,8 @@ def attach_file(report, path, key=None, overwrite=True):
     if not overwrite:
         while key in report:
             key += '_'
-    report[key] = read_file(path)
+    report[key] = read_file(path, force_unicode=force_unicode)
+
 
 def attach_conffiles(report, package, conffiles=None, ui=None):
     '''Attach information about any modified or deleted conffiles.
@@ -127,6 +147,7 @@ def attach_conffiles(report, package, conffiles=None, ui=None):
         mtime = datetime.datetime.fromtimestamp(os.stat(path).st_mtime)
         report['mtime.conffile.' + path_to_key(path)] = mtime.isoformat()
 
+
 def attach_upstart_overrides(report, package):
     '''Attach information about any Upstart override files'''
 
@@ -141,6 +162,7 @@ def attach_upstart_overrides(report, package):
             key = 'upstart.' + override.replace('/etc/init/', '')
             attach_file_if_exists(report, override, key)
 
+
 def attach_dmesg(report):
     '''Attach information from the kernel ring buffer (dmesg).
 
@@ -155,11 +177,12 @@ def attach_dmesg(report):
     if not report.get('CurrentDmesg', '').strip():
         dmesg = command_output(['sh', '-c', 'dmesg | comm -13 --nocheck-order /var/log/dmesg -'])
         # if an initial message was truncated by the ring buffer, skip over it
-        first_newline = dmesg.find(b'\n[')
+        first_newline = dmesg.find('\n[')
         if first_newline != -1:
-            dmesg = dmesg[first_newline+1:]
+            dmesg = dmesg[first_newline + 1:]
         report['CurrentDmesg'] = dmesg
 
+
 def attach_dmi(report):
     dmi_dir = '/sys/class/dmi/id'
     if os.path.isdir(dmi_dir):
@@ -180,6 +203,7 @@ def attach_dmi(report):
             if value:
                 report['dmi.' + f.replace('_', '.')] = value
 
+
 def attach_hardware(report):
     '''Attach a standard set of hardware-related data to the report, including:
 
@@ -200,30 +224,30 @@ def attach_hardware(report):
     attach_file(report, '/proc/interrupts', 'ProcInterrupts')
     attach_file(report, '/proc/cpuinfo', 'ProcCpuinfo')
     attach_file(report, '/proc/cmdline', 'ProcKernelCmdLine')
-    attach_file(report, '/var/log/udev', 'UdevLog')
+    attach_file(report, '/var/log/udev', 'UdevLog', force_unicode=True)
 
     if os.path.exists('/sys/bus/pci'):
-        report['Lspci'] = command_output(['lspci','-vvnn'])
+        report['Lspci'] = command_output(['lspci', '-vvnn'])
     report['Lsusb'] = command_output(['lsusb'])
     report['ProcModules'] = command_output(['sort', '/proc/modules'])
     report['UdevDb'] = command_output(['udevadm', 'info', '--export-db'])
 
     # anonymize partition labels
-    l = report['UdevLog'].decode('UTF-8', errors='replace')
+    l = report['UdevLog']
     l = re.sub('ID_FS_LABEL=(.*)', 'ID_FS_LABEL=<hidden>', l)
     l = re.sub('ID_FS_LABEL_ENC=(.*)', 'ID_FS_LABEL_ENC=<hidden>', l)
     l = re.sub('by-label/(.*)', 'by-label/<hidden>', l)
     l = re.sub('ID_FS_LABEL=(.*)', 'ID_FS_LABEL=<hidden>', l)
     l = re.sub('ID_FS_LABEL_ENC=(.*)', 'ID_FS_LABEL_ENC=<hidden>', l)
     l = re.sub('by-label/(.*)', 'by-label/<hidden>', l)
-    report['UdevLog'] = l.encode('UTF-8')
+    report['UdevLog'] = l
 
     attach_dmi(report)
 
     # Use the hardware information to create a machine type.
     if 'dmi.sys.vendor' in report and 'dmi.product.name' in report:
         report['MachineType'] = '%s %s' % (report['dmi.sys.vendor'],
-                report['dmi.product.name'])
+                                           report['dmi.product.name'])
 
     if command_available('prtconf'):
         report['Prtconf'] = command_output(['prtconf'])
@@ -236,10 +260,10 @@ def attach_hardware(report):
         if out:
             report['PccardctlIdent'] = out
 
-def attach_alsa(report):
-    '''Attach ALSA subsystem information to the report.
 
-    (loosely based on http://www.alsa-project.org/alsa-info.sh)
+def attach_alsa_old(report):
+    ''' (loosely based on http://www.alsa-project.org/alsa-info.sh)
+    for systems where alsa-info is not installed (i e, *buntu 12.04 and earlier)
     '''
     attach_file_if_exists(report, os.path.expanduser('~/.asoundrc'),
                           'UserAsoundrc')
@@ -249,9 +273,9 @@ def attach_alsa(report):
     attach_file_if_exists(report, '/proc/asound/version', 'AlsaVersion')
     attach_file(report, '/proc/cpuinfo', 'ProcCpuinfo')
 
-    report['AlsaDevices'] = command_output(['ls','-l','/dev/snd/'])
-    report['AplayDevices'] = command_output(['aplay','-l'])
-    report['ArecordDevices'] = command_output(['arecord','-l'])
+    report['AlsaDevices'] = command_output(['ls', '-l', '/dev/snd/'])
+    report['AplayDevices'] = command_output(['aplay', '-l'])
+    report['ArecordDevices'] = command_output(['arecord', '-l'])
 
     report['PciMultimedia'] = pci_devices(PCI_MULTIMEDIA)
 
@@ -281,10 +305,17 @@ def attach_alsa(report):
                     key = 'Card%d.Codecs.%s.%s' % (card, path_to_key(codec), path_to_key(name))
                     attach_file(report, path, key)
 
+
+def attach_alsa(report):
+    '''Attach ALSA subsystem information to the report.
+    '''
+    if os.path.exists('/usr/share/alsa-base/alsa-info.sh'):
+        report['AlsaInfo'] = command_output(['/usr/share/alsa-base/alsa-info.sh', '--stdout', '--no-upload'])
+    else:
+        attach_alsa_old(report)
+
     report['AudioDevicesInUse'] = command_output(
-        ['fuser','-v'] + glob.glob('/dev/dsp*')
-            + glob.glob('/dev/snd/*')
-            + glob.glob('/dev/seq*') )
+        ['fuser', '-v'] + glob.glob('/dev/dsp*') + glob.glob('/dev/snd/*') + glob.glob('/dev/seq*'))
 
     if os.path.exists('/usr/bin/pacmd'):
         report['PulseList'] = command_output(['pacmd', 'list'])
@@ -292,8 +323,6 @@ def attach_alsa(report):
     attach_dmi(report)
     attach_dmesg(report)
 
-    # This seems redundant with the amixer info, do we need it?
-    #report['AlsactlStore'] = command-output(['alsactl', '-f', '-', 'store'])
 
 def command_available(command):
     '''Is given command on the executable search path?'''
@@ -308,62 +337,61 @@ def command_available(command):
             return True
     return False
 
-def command_output(command, input=None, stderr=subprocess.STDOUT, keep_locale=False):
+
+def command_output(command, input=None, stderr=subprocess.STDOUT,
+                   keep_locale=False, decode_utf8=True):
     '''Try to execute given command (array) and return its stdout.
 
     In case of failure, a textual error gets returned. This function forces
     LC_MESSAGES to C, to avoid translated output in bug reports.
+
+    If decode_utf8 is True (default), the output will be converted to a string,
+    otherwise left as bytes.
     '''
     env = os.environ.copy()
     if not keep_locale:
         env['LC_MESSAGES'] = 'C'
     try:
-       sp = subprocess.Popen(command, stdout=subprocess.PIPE,
-                             stderr=stderr, 
-                             stdin=(input and subprocess.PIPE or None),
-                             close_fds=True, env=env)
+        sp = subprocess.Popen(command, stdout=subprocess.PIPE,
+                              stderr=stderr,
+                              stdin=(input and subprocess.PIPE or None),
+                              env=env)
     except OSError as e:
-       return 'Error: ' + str(e)
+        return 'Error: ' + str(e)
 
     out = sp.communicate(input)[0]
     if sp.returncode == 0:
-       return out.strip()
+        res = out.strip()
     else:
-       return 'Error: command %s failed with exit code %i: %s' % (
-           str(command), sp.returncode, out)
+        res = (b'Error: command ' + str(command).encode() + b' failed with exit code '
+               + str(sp.returncode).encode() + b': ' + out)
+
+    if decode_utf8:
+        res = res.decode('UTF-8', errors='replace')
+    return res
+
 
 def _root_command_prefix():
     if os.getuid() == 0:
-        prefix = []
-    elif os.getenv('DISPLAY') and \
-            subprocess.call(['which', 'kdesudo'], stdout=subprocess.PIPE,
-                    stderr=subprocess.PIPE) == 0 and \
-            subprocess.call(['pgrep', '-x', '-u', str(os.getuid()), 'ksmserver'],
-                    stdout=subprocess.PIPE, stderr=subprocess.PIPE) == 0:
-        prefix = ['kdesudo', '--desktop', '/usr/share/applications/apport-kde-mime.desktop', 
-                  '--', 'env', '-u', 'LANGUAGE', 'LC_MESSAGES=C']
-    elif os.getenv('DISPLAY') and \
-            subprocess.call(['which', 'gksu'], stdout=subprocess.PIPE,
-                    stderr=subprocess.PIPE) == 0 and \
-            subprocess.call(['pgrep', '-x', '-u', str(os.getuid()), 'gnome-panel|gconfd-2'],
-                    stdout=subprocess.PIPE, stderr=subprocess.PIPE) == 0:
-        prefix = ['gksu', '-D', 'Apport', '--', 'env', '-u', 'LANGUAGE', 'LC_MESSAGES=C']
+        return []
     else:
-        prefix = ['sudo', 'LC_MESSAGES=C', 'LANGUAGE=']
+        return ['pkexec']
 
-    return prefix
 
-def root_command_output(command, input=None, stderr=subprocess.STDOUT):
+def root_command_output(command, input=None, stderr=subprocess.STDOUT, decode_utf8=True):
     '''Try to execute given command (array) as root and return its stdout.
 
-    This passes the command through gksu, kdesudo, or sudo, depending on the
-    running desktop environment.
+    This passes the command through pkexec, unless the caller is already root.
 
     In case of failure, a textual error gets returned.
+
+    If decode_utf8 is True (default), the output will be converted to a string,
+    otherwise left as bytes.
     '''
-    assert type(command) == type([]), 'command must be a list'
+    assert isinstance(command, list), 'command must be a list'
     return command_output(_root_command_prefix() + command, input, stderr,
-            keep_locale=True)
+                          keep_locale=True, decode_utf8=decode_utf8)
+
 
 def attach_root_command_outputs(report, command_map):
     '''Execute multiple commands as root and put their outputs into report.
@@ -373,12 +401,14 @@ def attach_root_command_outputs(report, command_map):
     escaping yourself. To include stderr output of a command, end it with
     "2>&1".
 
-    Just like root_command_output() this will use gksu, kdesudo, or sudo for
-    gaining root privileges, depending on the running desktop environment.
+    Just like root_command_output, this passes the command through pkexec,
+    unless the caller is already root.
 
     This is preferrable to using root_command_output() multiple times, as that
     will ask for the password every time.
     '''
+    wrapper_path = os.path.join(os.path.abspath(
+        os.environ.get('APPORT_DATA_DIR', '/usr/share/apport')), 'root_info_wrapper')
     workdir = tempfile.mkdtemp()
     try:
         # create a shell script with all the commands
@@ -392,20 +422,25 @@ def attach_root_command_outputs(report, command_map):
         script.close()
 
         # run script
-        sp = subprocess.Popen(_root_command_prefix() + ['/bin/sh', script_path],
-            close_fds=True)
+        sp = subprocess.Popen(_root_command_prefix() + [wrapper_path, script_path])
         sp.wait()
 
         # now read back the individual outputs
         for keyname in command_map:
-            f = open(os.path.join(workdir, keyname))
-            buf = f.read().strip()
+            try:
+                with open(os.path.join(workdir, keyname)) as f:
+                    buf = f.read().strip()
+            except IOError:
+                # this can happen if the user dismisses authorization in
+                # _root_command_prefix
+                continue
             if buf:
                 report[keyname] = buf
             f.close()
     finally:
         shutil.rmtree(workdir)
 
+
 def recent_syslog(pattern):
     '''Extract recent messages from syslog which match a regex.
 
@@ -413,21 +448,29 @@ def recent_syslog(pattern):
     '''
     return recent_logfile('/var/log/syslog', pattern)
 
-def recent_logfile(logfile, pattern):
+
+def recent_logfile(logfile, pattern, maxlines=10000):
     '''Extract recent messages from a logfile which match a regex.
 
-    pattern should be a "re" object.
+    pattern should be a "re" object. By default this catches at most the last
+    1000 lines, but this can be modified with a different maxlines argument.
     '''
     lines = ''
     try:
-        with open(logfile) as f:
-            for line in f:
+        tail = subprocess.Popen(['tail', '-n', str(maxlines), logfile],
+                                stdout=subprocess.PIPE)
+        while tail.poll() is None:
+            for line in tail.stdout:
+                line = line.decode('UTF-8', errors='replace')
                 if pattern.search(line):
                     lines += line
+        tail.stdout.close()
+        tail.wait()
     except IOError:
         return ''
     return lines
 
+
 def xsession_errors(pattern=None):
     '''Extract messages from ~/.xsession-errors.
 
@@ -446,9 +489,11 @@ def xsession_errors(pattern=None):
         pattern = re.compile('^(\(.*:\d+\): \w+-(WARNING|CRITICAL|ERROR))|(Error: .*No Symbols named)|([^ ]+\[\d+\]: ([A-Z]+):)|([^ ]-[A-Z]+ \*\*:)|(received an X Window System error)|(^The error was \')|(^  \(Details: serial \d+ error_code)')
 
     lines = ''
-    for line in open(path):
-        if pattern.search(line):
-            lines += line
+    with open(path, 'rb') as f:
+        for line in f:
+            line = line.decode('UTF-8', errors='replace')
+            if pattern.search(line):
+                lines += line
     return lines
 
 PCI_MASS_STORAGE = 0x01
@@ -464,6 +509,7 @@ PCI_DOCKING_STATIONS = 0x0a
 PCI_PROCESSORS = 0x0b
 PCI_SERIAL_BUS = 0x0c
 
+
 def pci_devices(*pci_classes):
     '''Return a text dump of PCI devices attached to the system.'''
 
@@ -471,20 +517,20 @@ def pci_devices(*pci_classes):
         return command_output(['lspci', '-vvnn'])
 
     result = ''
-    output = command_output(['lspci','-vvmmnn'])
-    for paragraph in output.split(b'\n\n'):
+    output = command_output(['lspci', '-vvmmnn'])
+    for paragraph in output.split('\n\n'):
         pci_class = None
         slot = None
 
-        for line in paragraph.split(b'\n'):
+        for line in paragraph.split('\n'):
             try:
-                key, value = line.split(b':',1)
+                key, value = line.split(':', 1)
             except ValueError:
                 continue
             value = value.strip()
             key = key.strip()
             if key == 'Class':
-                n = int(value[-5:-1],16)
+                n = int(value[-5:-1], 16)
                 pci_class = (n & 0xff00) >> 8
             elif key == 'Slot':
                 slot = value
@@ -496,11 +542,13 @@ def pci_devices(*pci_classes):
 
     return result
 
+
 def usb_devices():
     '''Return a text dump of USB devices attached to the system.'''
 
     # TODO: would be nice to be able to filter by interface class
-    return command_output(['lsusb','-v'])
+    return command_output(['lsusb', '-v'])
+
 
 def files_in_package(package, globpat=None):
     '''Retrieve a list of files owned by package, optionally matching globpat'''
@@ -512,57 +560,60 @@ def files_in_package(package, globpat=None):
         result = files
     return result
 
+
 def attach_gconf(report, package):
     '''Obsolete'''
 
     # keeping a no-op function for some time to not break hooks
     pass
 
+
 def attach_gsettings_schema(report, schema):
     '''Attach user-modified gsttings keys of a schema.'''
 
     cur_value = report.get('GsettingsChanges', '')
 
-    defaults = {} # schema -> key ->  value
+    defaults = {}  # schema -> key ->  value
     env = os.environ.copy()
     env['XDG_CONFIG_HOME'] = '/nonexisting'
     gsettings = subprocess.Popen(['gsettings', 'list-recursively', schema],
-            env=env, stdout=subprocess.PIPE)
+                                 env=env, stdout=subprocess.PIPE)
     for l in gsettings.stdout:
         try:
             (schema, key, value) = l.split(None, 2)
             value = value.rstrip()
         except ValueError:
-            continue # invalid line
+            continue  # invalid line
         defaults.setdefault(schema, {})[key] = value
 
     gsettings = subprocess.Popen(['gsettings', 'list-recursively', schema],
-            stdout=subprocess.PIPE)
+                                 stdout=subprocess.PIPE)
     for l in gsettings.stdout:
         try:
             (schema, key, value) = l.split(None, 2)
             value = value.rstrip()
         except ValueError:
-            continue # invalid line
+            continue  # invalid line
 
         if value != defaults.get(schema, {}).get(key, ''):
             cur_value += '%s %s %s\n' % (schema, key, value)
 
     report['GsettingsChanges'] = cur_value
 
+
 def attach_gsettings_package(report, package):
     '''Attach user-modified gsettings keys of all schemas in a package.'''
 
-    for schema_file in files_in_package(package,
-            '/usr/share/glib-2.0/schemas/*.gschema.xml'):
+    for schema_file in files_in_package(package, '/usr/share/glib-2.0/schemas/*.gschema.xml'):
         schema = os.path.basename(schema_file)[:-12]
         attach_gsettings_schema(report, schema)
 
+
 def attach_network(report):
     '''Attach generic network-related information to report.'''
 
-    report['IpRoute'] = command_output(['ip','route'])
-    report['IpAddr'] = command_output(['ip','addr'])
+    report['IpRoute'] = command_output(['ip', 'route'])
+    report['IpAddr'] = command_output(['ip', 'addr'])
     report['PciNetwork'] = pci_devices(PCI_NETWORK)
     attach_file_if_exists(report, '/etc/network/interfaces', key='IfupdownConfig')
 
@@ -570,19 +621,22 @@ def attach_network(report):
         if var in os.environ:
             report[var] = os.environ[var]
 
+
 def attach_wifi(report):
     '''Attach wireless (WiFi) network information to report.'''
 
     report['WifiSyslog'] = recent_syslog(re.compile(r'(NetworkManager|modem-manager|dhclient|kernel|wpa_supplicant)(\[\d+\])?:'))
-    report['IwConfig'] = re.sub('ESSID:(.*)', 'ESSID:<hidden>',
+    report['IwConfig'] = re.sub(
+        'ESSID:(.*)', 'ESSID:<hidden>',
         re.sub('Encryption key:(.*)', 'Encryption key: <hidden>',
-        re.sub('Access Point: (.*)', 'Access Point: <hidden>',
-            command_output(['iwconfig']).decode('UTF-8', errors='ignore'))))
+               re.sub('Access Point: (.*)', 'Access Point: <hidden>',
+                      command_output(['iwconfig']))))
     report['RfKill'] = command_output(['rfkill', 'list'])
     report['CRDA'] = command_output(['iw', 'reg', 'get'])
 
     attach_file_if_exists(report, '/var/log/wpa_supplicant.log', key='WpaSupplicantLog')
 
+
 def attach_printing(report):
     '''Attach printing information to the report.
 
@@ -609,30 +663,59 @@ def attach_printing(report):
         'cupsys-driver-gutenprint', 'gimp-gutenprint', 'gutenprint-doc',
         'gutenprint-locales', 'system-config-printer-common', 'kdeprint')
 
-def attach_mac_events(report):
+
+def attach_mac_events(report, profiles=None):
     '''Attach MAC information and events to the report.'''
 
+    # Allow specifying a string, or a list of strings
+    if isinstance(profiles, str):
+        profiles = [profiles]
+
     mac_regex = 'audit\(|apparmor|selinux|security'
     mac_re = re.compile(mac_regex, re.IGNORECASE)
-    aa_denied_regex = 'apparmor="DENIED"'
-    aa_denied_re = re.compile(aa_denied_regex, re.IGNORECASE)
+    aa_regex = 'apparmor="DENIED".+?profile=([^ ]+?)[ ]'
+    aa_re = re.compile(aa_regex, re.IGNORECASE)
 
-    if os.path.exists('/var/log/kern.log'):
-        report['KernLog'] = recent_logfile('/var/log/kern.log', mac_re)
-    elif os.path.exists('/var/log/messages'):
-        report['KernLog'] = recent_logfile('/var/log/messages', mac_re)
+    if 'KernLog' not in report:
+        if os.path.exists('/var/log/kern.log'):
+            report['KernLog'] = recent_logfile('/var/log/kern.log', mac_re)
+        elif os.path.exists('/var/log/messages'):
+            report['KernLog'] = recent_logfile('/var/log/messages', mac_re)
 
-    if os.path.exists('/var/run/auditd.pid'):
+    if 'AuditLog' not in report and os.path.exists('/var/run/auditd.pid'):
         attach_root_command_outputs(report, {'AuditLog': 'egrep "' + mac_regex + '" /var/log/audit/audit.log'})
 
     attach_file(report, '/proc/version_signature', 'ProcVersionSignature')
     attach_file(report, '/proc/cmdline', 'ProcCmdline')
 
-    if re.search(aa_denied_re, report.get('KernLog', '')) or re.search(aa_denied_re, report.get('AuditLog', '')):
-        tags = report.get('Tags', '')
-        if tags:
-            tags += ' '
-        report['Tags'] = tags + 'apparmor'
+    for match in re.findall(aa_re, report.get('KernLog', '') + report.get('AuditLog', '')):
+        if not profiles:
+            _add_tag(report, 'apparmor')
+            break
+
+        try:
+            if match[0] == '"':
+                profile = match[1:-1]
+            elif sys.version[0] >= '3':
+                profile = bytes.fromhex(match).decode('UTF-8', errors='replace')
+            else:
+                profile = match.decode('hex', errors='replace')
+        except:
+            continue
+
+        for search_profile in profiles:
+            if re.match('^' + search_profile + '$', profile):
+                _add_tag(report, 'apparmor')
+                break
+
+
+def _add_tag(report, tag):
+    '''Adds or appends a tag to the report'''
+    current_tags = report.get('Tags', '')
+    if current_tags:
+        current_tags += ' '
+    report['Tags'] = current_tags + tag
+
 
 def attach_related_packages(report, packages):
     '''Attach version information for related packages
@@ -641,6 +724,7 @@ def attach_related_packages(report, packages):
     '''
     report['RelatedPackageVersions'] = package_versions(*packages)
 
+
 def package_versions(*packages):
     '''Return a text listing of package names and versions.
 
@@ -663,7 +747,7 @@ def package_versions(*packages):
                 version = 'N/A'
             if version is None:
                 version = 'N/A'
-            versions.append((package,version))
+            versions.append((package, version))
 
     package_width, version_width = \
         map(max, [map(len, t) for t in zip(*versions)])
@@ -671,6 +755,7 @@ def package_versions(*packages):
     fmt = '%%-%ds %%s' % package_width
     return '\n'.join([fmt % v for v in versions])
 
+
 def shared_libraries(path):
     '''Returns a list of strings containing the sonames of shared libraries
     with which the specified binary is linked.'''
@@ -688,6 +773,7 @@ def shared_libraries(path):
 
     return libs
 
+
 def links_with_shared_library(path, lib):
     '''Returns True if the binary at path links with the library named lib.
 
@@ -697,19 +783,22 @@ def links_with_shared_library(path, lib):
 
     libs = shared_libraries(path)
 
-    if lib in libs: return True
+    if lib in libs:
+        return True
 
     for linked_lib in libs:
-        if linked_lib.startswith(lib + '.so.'): return True
+        if linked_lib.startswith(lib + '.so.'):
+            return True
 
     return False
 
+
 def _get_module_license(module):
     '''Return the license for a given kernel module.'''
 
     try:
         modinfo = subprocess.Popen(['/sbin/modinfo', module],
-                stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+                                   stdout=subprocess.PIPE, stderr=subprocess.PIPE)
         out = modinfo.communicate()[0].decode('UTF-8')
         if modinfo.returncode != 0:
             return 'invalid'
@@ -724,7 +813,8 @@ def _get_module_license(module):
 
     return None
 
-def nonfree_kernel_modules(module_list = '/proc/modules'):
+
+def nonfree_kernel_modules(module_list='/proc/modules'):
     '''Check loaded modules and return a list of those which are not free.'''
 
     try:
@@ -741,6 +831,7 @@ def nonfree_kernel_modules(module_list = '/proc/modules'):
 
     return nonfree
 
+
 def __drm_con_info(con):
     info = ''
     for f in os.listdir(con):
@@ -757,6 +848,7 @@ def __drm_con_info(con):
         info += '%s: %s\n' % (f, val)
     return info
 
+
 def attach_drm_info(report):
     '''Add information about DRM hardware.
 
@@ -771,6 +863,7 @@ def attach_drm_info(report):
             # DRM can set an arbitrary string for its connector paths.
             report['DRM.' + path_to_key(f)] = __drm_con_info(con)
 
+
 def in_session_of_problem(report):
     '''Check if the problem happened in the currently running XDG session.
 
@@ -792,13 +885,15 @@ def in_session_of_problem(report):
 
     try:
         bus = Gio.bus_get_sync(Gio.BusType.SYSTEM, None)
-        ck_manager = Gio.DBusProxy.new_sync(bus, Gio.DBusProxyFlags.NONE, None,
+        ck_manager = Gio.DBusProxy.new_sync(
+            bus, Gio.DBusProxyFlags.NONE, None,
             'org.freedesktop.ConsoleKit', '/org/freedesktop/ConsoleKit/Manager',
             'org.freedesktop.ConsoleKit.Manager', None)
 
         cur_session = ck_manager.GetCurrentSession()
 
-        ck_session = Gio.DBusProxy.new_sync(bus, Gio.DBusProxyFlags.NONE, None,
+        ck_session = Gio.DBusProxy.new_sync(
+            bus, Gio.DBusProxyFlags.NONE, None,
             'org.freedesktop.ConsoleKit', cur_session,
             'org.freedesktop.ConsoleKit.Session', None)
 
@@ -807,8 +902,7 @@ def in_session_of_problem(report):
         sys.stderr.write('Error connecting to ConsoleKit: %s\n' % str(e))
         return None
 
-    m = re.match('(\d{4}-\d\d-\d\dT\d\d:\d\d:\d\d)(?:\.\d+Z)$',
-            session_start_time)
+    m = re.match('(\d{4}-\d\d-\d\dT\d\d:\d\d:\d\d)(?:\.\d+Z)$', session_start_time)
     if m:
         # CK gives UTC time
         session_start_time = calendar.timegm(time.strptime(m.group(1), '%Y-%m-%dT%H:%M:%S'))
@@ -817,3 +911,18 @@ def in_session_of_problem(report):
         return None
 
     return session_start_time <= report_time
+
+
+def attach_default_grub(report, key=None):
+    '''attach /etc/default/grub after filtering out password lines'''
+
+    path = '/etc/default/grub'
+    if not key:
+        key = path_to_key(path)
+
+    if os.path.exists(path):
+        with open(path, 'r') as f:
+            filtered = [l if not l.startswith('password')
+                        else '### PASSWORD LINE REMOVED ###'
+                        for l in f.readlines()]
+            report[key] = ''.join(filtered)
