@@ -12,22 +12,26 @@
 
 import os
 import sys
+import re
+from glob import glob
 
 CONFIG = '/etc/default/apport'
 
-# This doesn't use apport.packaging.enabled() because it is too heavyweight
-# See LP: #528355
+
 def enabled():
     '''Return whether Apport should generate crash reports.'''
 
-    import re
+    # This doesn't use apport.packaging.enabled() because it is too heavyweight
+    # See LP: #528355
     try:
-        conf = open(CONFIG).read()
+        with open(CONFIG) as f:
+            conf = f.read()
         return re.search('^\s*enabled\s*=\s*0\s*$', conf, re.M) is None
     except IOError:
         # if the file does not exist, assume it's enabled
         return True
 
+
 def apport_excepthook(exc_type, exc_obj, exc_tb):
     '''Catch an uncaught exception and make a traceback.'''
 
@@ -50,13 +54,9 @@ def apport_excepthook(exc_type, exc_obj, exc_tb):
         if not enabled():
             return
 
-        # org.freedesktop.DBus.Error.NoReply is an useless crash, needs actual
-        # crash from D-BUS backend (LP# 914220)
-        if 'org.freedesktop.DBus.Error.NoReply' in str(exc_obj):
-            return
-
         try:
             from cStringIO import StringIO
+            StringIO  # pyflakes
         except ImportError:
             from io import StringIO
 
@@ -65,7 +65,7 @@ def apport_excepthook(exc_type, exc_obj, exc_tb):
 
         # apport will look up the package from the executable path.
         try:
-            binary = os.path.realpath(os.path.join(os.getcwdu(), sys.argv[0]))
+            binary = os.path.realpath(os.path.join(os.getcwd(), sys.argv[0]))
         except (TypeError, AttributeError, IndexError):
             # the module has mutated sys.argv, plan B
             try:
@@ -85,14 +85,24 @@ def apport_excepthook(exc_type, exc_obj, exc_tb):
         import apport.report
 
         pr = apport.report.Report()
+
+        # special handling of dbus-python exceptions
+        if hasattr(exc_obj, 'get_dbus_name'):
+            if exc_obj.get_dbus_name() == 'org.freedesktop.DBus.Error.NoReply':
+                # NoReply is an useless crash, we do not even get the method it
+                # was trying to call; needs actual crash from D-BUS backend (LP #914220)
+                return
+            if exc_obj.get_dbus_name() == 'org.freedesktop.DBus.Error.ServiceUnknown':
+                dbus_service_unknown_analysis(exc_obj, pr)
+
         # append a basic traceback. In future we may want to include
         # additional data such as the local variables, loaded modules etc.
         tb_file = StringIO()
         traceback.print_exception(exc_type, exc_obj, exc_tb, file=tb_file)
         pr['Traceback'] = tb_file.getvalue().strip()
-        pr.add_proc_info()
+        pr.add_proc_info(extraenv=['PYTHONPATH', 'PYTHONHOME'])
         pr.add_user_info()
-        # override the ExecutablePath with the script that was actually running.
+        # override the ExecutablePath with the script that was actually running
         pr['ExecutablePath'] = binary
         try:
             pr['PythonArgs'] = '%r' % sys.argv
@@ -103,13 +113,14 @@ def apport_excepthook(exc_type, exc_obj, exc_tb):
         mangled_program = re.sub('/', '_', binary)
         # get the uid for now, user name later
         user = os.getuid()
-        pr_filename = '%s/%s.%i.crash' % (os.environ.get('APPORT_REPORT_DIR',
-            '/var/crash'), mangled_program, user)
+        pr_filename = '%s/%s.%i.crash' % (os.environ.get(
+            'APPORT_REPORT_DIR', '/var/crash'), mangled_program, user)
         crash_counter = 0
         if os.path.exists(pr_filename):
             if apport.fileutils.seen_report(pr_filename):
                 # flood protection
-                crash_counter = get_recent_crashes(open(pr_filename)) + 1
+                with open(pr_filename, 'rb') as f:
+                    crash_counter = get_recent_crashes(f) + 1
                 if crash_counter > 1:
                     return
 
@@ -122,12 +133,9 @@ def apport_excepthook(exc_type, exc_obj, exc_tb):
 
         if crash_counter:
             pr['CrashCounter'] = str(crash_counter)
-        report_file = os.fdopen(os.open(pr_filename,
-            os.O_WRONLY|os.O_CREAT|os.O_EXCL, 0o640), 'w')
-        try:
-            pr.write(report_file)
-        finally:
-            report_file.close()
+        with os.fdopen(os.open(pr_filename,
+                               os.O_WRONLY | os.O_CREAT | os.O_EXCL, 0o640), 'wb') as f:
+            pr.write(f)
 
     finally:
         # resume original processing to get the default behaviour,
@@ -136,8 +144,52 @@ def apport_excepthook(exc_type, exc_obj, exc_tb):
             sys.__excepthook__(exc_type, exc_obj, exc_tb)
 
 
+def dbus_service_unknown_analysis(exc_obj, report):
+    import subprocess
+    try:
+        from configparser import ConfigParser, NoSectionError, NoOptionError
+        (ConfigParser, NoSectionError, NoOptionError)  # pyflakes
+    except ImportError:
+        # Python 2
+        from ConfigParser import ConfigParser, NoSectionError, NoOptionError
+
+    # determine D-BUS name
+    m = re.search('name\s+(\S+)\s+was not provided by any .service',
+                  exc_obj.get_dbus_message())
+    if not m:
+        if sys.stderr:
+            sys.stderr.write('Error: cannot parse D-BUS name from exception: '
+                             + exc_obj.get_dbus_message())
+            return
+
+    dbus_name = m.group(1)
+
+    # determine .service file and Exec name for the D-BUS name
+    services = []  # tuples of (service file, exe name, running)
+    for f in glob('/usr/share/dbus-1/*services/*.service'):
+        cp = ConfigParser(interpolation=None)
+        cp.read(f, encoding='UTF-8')
+        try:
+            if cp.get('D-BUS Service', 'Name') == dbus_name:
+                exe = cp.get('D-BUS Service', 'Exec')
+                running = (subprocess.call(['pidof', '-sx', exe], stdout=subprocess.PIPE) == 0)
+                services.append((f, exe, running))
+        except (NoSectionError, NoOptionError):
+            if sys.stderr:
+                sys.stderr.write('Invalid D-BUS .service file %s: %s' % (
+                    f, exc_obj.get_dbus_message()))
+            continue
+
+    if not services:
+        report['DbusErrorAnalysis'] = 'no service file providing ' + dbus_name
+    else:
+        report['DbusErrorAnalysis'] = 'provided by'
+        for (service, exe, running) in services:
+            report['DbusErrorAnalysis'] += ' %s (%s is %srunning)' % (
+                service, exe, ('' if running else 'not '))
+
+
 def install():
     '''Install the python apport hook.'''
 
     sys.excepthook = apport_excepthook
-
