@@ -18,10 +18,19 @@ import hashlib
 import warnings
 warnings.filterwarnings('ignore', 'apt API not stable yet', FutureWarning)
 import apt
+try:
+    import cPickle as pickle
+    from urllib import urlopen
+    (pickle, urlopen)  # pyflakes
+except ImportError:
+    # python 3
+    from urllib.request import urlopen
+    import pickle
 
 import apport
 from apport.packaging import PackageInfo
 
+
 class __AptDpkgPackageInfo(PackageInfo):
     '''Concrete apport.PackageInfo class implementation for python-apt and
     dpkg, as found on Debian and derivatives such as Ubuntu.'''
@@ -31,17 +40,36 @@ class __AptDpkgPackageInfo(PackageInfo):
         self._sandbox_apt_cache = None
         self._contents_dir = None
         self._mirror = None
+        self._virtual_mapping_obj = None
 
         self.configuration = '/etc/default/apport'
 
     def __del__(self):
         try:
             if self._contents_dir:
-                import shutil
                 shutil.rmtree(self._contents_dir)
         except AttributeError:
             pass
 
+    def _virtual_mapping(self, configdir):
+        if self._virtual_mapping_obj is not None:
+            return self._virtual_mapping_obj
+
+        mapping_file = os.path.join(configdir, 'virtual_mapping.pickle')
+        if os.path.exists(mapping_file):
+            with open(mapping_file, 'rb') as fp:
+                self._virtual_mapping_obj = pickle.load(fp)
+        else:
+            self._virtual_mapping_obj = {}
+
+        return self._virtual_mapping_obj
+
+    def _save_virtual_mapping(self, configdir):
+        mapping_file = os.path.join(configdir, 'virtual_mapping.pickle')
+        if self._virtual_mapping_obj is not None:
+            with open(mapping_file, 'wb') as fp:
+                pickle.dump(self._virtual_mapping_obj, fp)
+
     def _cache(self):
         '''Return apt.Cache() (initialized lazily).'''
 
@@ -84,7 +112,7 @@ class __AptDpkgPackageInfo(PackageInfo):
         try:
             return self._cache()[package]
         except KeyError:
-            raise ValueError('package does not exist')
+            raise ValueError('package %s does not exist' % package)
 
     def get_version(self, package):
         '''Return the installed version of a package.'''
@@ -92,7 +120,7 @@ class __AptDpkgPackageInfo(PackageInfo):
         pkg = self._apt_pkg(package)
         inst = pkg.installed
         if not inst:
-            raise ValueError('package does not exist')
+            raise ValueError('package %s does not exist' % package)
         return inst.version
 
     def get_available_version(self, package):
@@ -107,8 +135,10 @@ class __AptDpkgPackageInfo(PackageInfo):
         if not cur_ver:
             # happens with virtual packages
             return []
-        return [d[0].target_pkg.name for d in cur_ver.depends_list.get('Depends', []) +
-            cur_ver.depends_list.get('PreDepends', [])]
+        return [d[0].target_pkg.name for d in
+                cur_ver.depends_list.get('Depends', []) +
+                cur_ver.depends_list.get('PreDepends', []) +
+                cur_ver.depends_list.get('Recommends', [])]
 
     def get_source(self, package):
         '''Return the source package name for a package.'''
@@ -141,7 +171,7 @@ class __AptDpkgPackageInfo(PackageInfo):
         a third-party source.'''
 
         lsb_release = subprocess.Popen(['lsb_release', '-i', '-s'],
-            stdout=subprocess.PIPE)
+                                       stdout=subprocess.PIPE)
         this_os = lsb_release.communicate()[0].decode().strip()
         assert lsb_release.returncode == 0
 
@@ -161,10 +191,8 @@ class __AptDpkgPackageInfo(PackageInfo):
             except IOError:
                 pass
 
-        origins = None
-        origins = pkg.candidate.origins
-        if origins: # might be None
-            for o in origins:
+        if pkg.candidate and pkg.candidate.origins:  # might be None
+            for o in pkg.candidate.origins:
                 if o.origin in native_origins:
                     return True
         return False
@@ -206,7 +234,7 @@ class __AptDpkgPackageInfo(PackageInfo):
             return []
 
         # create a list of files with a newer timestamp for md5sum'ing
-        sums = ''
+        sums = b''
         sumfile = '/var/lib/dpkg/info/%s:%s.md5sums' % (package, self.get_system_architecture())
         if not os.path.exists(sumfile):
             sumfile = '/var/lib/dpkg/info/%s.md5sums' % package
@@ -214,18 +242,18 @@ class __AptDpkgPackageInfo(PackageInfo):
                 # some packages do not ship md5sums
                 return []
 
-        with open(sumfile) as fd:
+        with open(sumfile, 'rb') as fd:
             for line in fd:
                 try:
                     # ignore lines with NUL bytes (happens, LP#96050)
-                    if '\0' in line:
+                    if b'\0' in line:
                         apport.warning('%s contains NUL character, ignoring line', sumfile)
                         continue
-                    words  = line.split()
+                    words = line.split()
                     if not words:
                         apport.warning('%s contains empty line, ignoring line', sumfile)
                         continue
-                    s = os.stat('/' + words[-1])
+                    s = os.stat('/' + words[-1].decode('UTF-8'))
                     if max(s.st_mtime, s.st_ctime) <= max_time:
                         continue
                 except OSError:
@@ -246,12 +274,12 @@ class __AptDpkgPackageInfo(PackageInfo):
         official user-facing API for this, which will ask for confirmation and
         allows filtering.
         '''
-        dpkg = subprocess.Popen(['dpkg-query','-W','--showformat=${Conffiles}',
-            package], stdout=subprocess.PIPE, close_fds=True)
+        dpkg = subprocess.Popen(['dpkg-query', '-W', '--showformat=${Conffiles}',
+                                 package], stdout=subprocess.PIPE)
 
         out = dpkg.communicate()[0].decode()
         if dpkg.returncode != 0:
-           return {}
+            return {}
 
         modified = {}
         for line in out.splitlines():
@@ -285,8 +313,8 @@ class __AptDpkgPackageInfo(PackageInfo):
 
         while not match and i < len(file_list):
             p = subprocess.Popen(['fgrep', '-lxm', '1', '--', pattern] +
-                file_list[i:i+slice_size], stdin=subprocess.PIPE,
-                stdout=subprocess.PIPE, stderr=subprocess.PIPE, close_fds=True)
+                                 file_list[i:(i + slice_size)], stdin=subprocess.PIPE,
+                                 stdout=subprocess.PIPE, stderr=subprocess.PIPE)
             out = p.communicate()[0].decode('UTF-8')
             if p.returncode == 0:
                 match = out
@@ -306,7 +334,7 @@ class __AptDpkgPackageInfo(PackageInfo):
         '''
         # check if the file is a diversion
         dpkg = subprocess.Popen(['/usr/sbin/dpkg-divert', '--list', file],
-            stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+                                stdout=subprocess.PIPE, stderr=subprocess.PIPE)
         out = dpkg.communicate()[0].decode('UTF-8')
         if dpkg.returncode == 0 and out:
             pkg = out.split()[-1]
@@ -343,7 +371,7 @@ class __AptDpkgPackageInfo(PackageInfo):
         particular distribution.'''
 
         dpkg = subprocess.Popen(['dpkg', '--print-architecture'],
-            stdout=subprocess.PIPE)
+                                stdout=subprocess.PIPE)
         arch = dpkg.communicate()[0].decode().strip()
         assert dpkg.returncode == 0
         assert arch
@@ -357,7 +385,7 @@ class __AptDpkgPackageInfo(PackageInfo):
         account.
         '''
         dpkg = subprocess.Popen(['dpkg-architecture', '-qDEB_HOST_MULTIARCH'],
-                stdout=subprocess.PIPE)
+                                stdout=subprocess.PIPE)
         multiarch_triple = dpkg.communicate()[0].decode().strip()
         assert dpkg.returncode == 0
 
@@ -372,27 +400,46 @@ class __AptDpkgPackageInfo(PackageInfo):
 
         self._mirror = url
 
-    def get_source_tree(self, srcpackage, dir, version=None):
-        '''Download given source package and unpack it into dir (which should
-        be empty).
+    def get_source_tree(self, srcpackage, dir, version=None, sandbox=None,
+                        apt_update=False):
+        '''Download source package and unpack it into dir.
 
         This also has to care about applying patches etc., so that dir will
-        eventually contain the actually compiled source.
+        eventually contain the actually compiled source. dir needs to exist and
+        should be empty.
 
         If version is given, this particular version will be retrieved.
         Otherwise this will fetch the latest available version.
 
+        If sandbox is given, it calls apt-get source in that sandbox, otherwise
+        it uses the system apt configuration.
+
+        If apt_update is True, it will call apt-get update before apt-get
+        source. This is mostly necessary for freshly created sandboxes.
+
         Return the directory that contains the actual source root directory
         (which might be a subdirectory of dir). Return None if the source is
-        not available.'''
+        not available.
+        '''
+        # configure apt for sandbox
+        env = os.environ.copy()
+        if sandbox:
+            f = tempfile.NamedTemporaryFile()
+            f.write(('''Dir "%s";
+Debug::NoLocking "true";
+ ''' % sandbox).encode())
+            f.flush()
+            env['APT_CONFIG'] = f.name
+
+        if apt_update:
+            subprocess.call(['apt-get', '-qq', 'update'], env=env)
 
         # fetch source tree
-        argv = ['apt-get', '--assume-yes', 'source', srcpackage]
+        argv = ['apt-get', '-qq', '--assume-yes', 'source', srcpackage]
         if version:
             argv[-1] += '=' + version
         try:
-            if subprocess.call(argv, stdout=subprocess.PIPE,
-                cwd=dir) != 0:
+            if subprocess.call(argv, cwd=dir, env=env) != 0:
                 return None
         except OSError:
             return None
@@ -406,11 +453,10 @@ class __AptDpkgPackageInfo(PackageInfo):
 
         # apply patches on a best-effort basis
         try:
-            subprocess.call('debian/rules patch || debian/rules apply-patches ' \
-                '|| debian/rules apply-dpatches || '\
-                'debian/rules unpack || debian/rules patch-stamp || ' \
-                'debian/rules setup', shell=True, cwd=root,
-                stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+            subprocess.call('(debian/rules patch || debian/rules apply-patches '
+                            '|| debian/rules apply-dpatches || '
+                            'debian/rules unpack || debian/rules patch-stamp || '
+                            'debian/rules setup) >/dev/null 2>&1', shell=True, cwd=root)
         except OSError:
             pass
 
@@ -431,7 +477,6 @@ class __AptDpkgPackageInfo(PackageInfo):
         special in various ways currently so we can not use the apt
         method.
         '''
-        import urllib, apt_pkg
         installed = []
         outdated = []
         kver = report['Uname'].split()[1]
@@ -442,17 +487,17 @@ class __AptDpkgPackageInfo(PackageInfo):
         if debug_pkgname in c and c[debug_pkgname].isInstalled:
             #print('kernel ddeb already installed')
             return (installed, outdated)
-        target_dir = apt_pkg.Config.FindDir('Dir::Cache::archives')+'/partial'
+        target_dir = apt.apt_pkg.config.find_dir('Dir::Cache::archives') + '/partial'
         deb = '%s_%s_%s.ddeb' % (debug_pkgname, ver, arch)
         # FIXME: this package is currently not in Packages.gz
         url = 'http://ddebs.ubuntu.com/pool/main/l/linux/%s' % deb
         out = open(os.path.join(target_dir, deb), 'w')
         # urlretrieve does not return 404 in the headers so we use urlopen
-        u = urllib.urlopen(url)
+        u = urlopen(url)
         if u.getcode() > 400:
             return ('', 'linux')
         while True:
-            block = u.read(8*1024)
+            block = u.read(8 * 1024)
             if not block:
                 break
             out.write(block)
@@ -463,7 +508,7 @@ class __AptDpkgPackageInfo(PackageInfo):
         return (installed, outdated)
 
     def install_packages(self, rootdir, configdir, release, packages,
-            verbose=False, cache_dir=None):
+                         verbose=False, cache_dir=None, permanent_rootdir=False):
         '''Install packages into a sandbox (for apport-retrace).
 
         In order to work without any special permissions and without touching
@@ -485,6 +530,9 @@ class __AptDpkgPackageInfo(PackageInfo):
         If cache_dir is given, then the downloaded packages will be stored
         there, to speed up subsequent retraces.
 
+        If permanent_rootdir is True, then the sandbox created from the
+        downloaded packages will be reused, to speed up subsequent retraces.
+
         Return a string with outdated packages, or None if all packages were
         installed.
 
@@ -506,10 +554,8 @@ class __AptDpkgPackageInfo(PackageInfo):
                 aptroot = os.path.join(cache_dir, release, 'apt')
             else:
                 aptroot = os.path.join(cache_dir, 'system', 'apt')
-            try:
+            if not os.path.isdir(aptroot):
                 os.makedirs(aptroot)
-            except OSError:
-                pass
         else:
             tmp_aptroot = True
             aptroot = tempfile.mkdtemp()
@@ -539,7 +585,7 @@ class __AptDpkgPackageInfo(PackageInfo):
             except KeyError:
                 candidate = None
             if not candidate:
-                m = 'package %s does not exist, ignoring' % pkg
+                m = 'package %s does not exist, ignoring' % pkg.replace('%', '%%')
                 obsolete += m + '\n'
                 apport.warning(m)
                 continue
@@ -549,6 +595,50 @@ class __AptDpkgPackageInfo(PackageInfo):
                 obsolete += w + '\n'
             real_pkgs.add(pkg)
 
+            if permanent_rootdir:
+                virtual_mapping = self._virtual_mapping(aptroot)
+                # Remember all the virtual packages that this package provides,
+                # so that if we encounter that virtual package as a
+                # Conflicts/Replaces later, we know to remove this package from
+                # the cache.
+                for p in candidate.provides:
+                    virtual_mapping.setdefault(p, set()).add(pkg)
+                conflicts = []
+                if 'Conflicts' in candidate.record:
+                    conflicts += apt.apt_pkg.parse_depends(candidate.record['Conflicts'])
+                if 'Replaces' in candidate.record:
+                    conflicts += apt.apt_pkg.parse_depends(candidate.record['Replaces'])
+                archives = apt.apt_pkg.config.find_dir('Dir::Cache::archives')
+                for conflict in conflicts:
+                    # apt_pkg.parse_depends needs to handle the or operator,
+                    # but as policy states it is invalid to use that in
+                    # Replaces/Depends, we can safely choose the first value
+                    # here.
+                    conflict = conflict[0]
+                    if c.is_virtual_package(conflict[0]):
+                        try:
+                            providers = virtual_mapping[conflict[0]]
+                        except KeyError:
+                            # We may not have seen the virtual package that
+                            # this conflicts with, so we can assume it's not
+                            # unpacked into the sandbox.
+                            continue
+                        for p in providers:
+                            debs = os.path.join(archives, '%s_*.deb' % p)
+                            for path in glob.glob(debs):
+                                ver = self._deb_version(path)
+                                if apt.apt_pkg.check_dep(ver, conflict[2],
+                                                              conflict[1]):
+                                    os.unlink(path)
+                        del providers
+                    else:
+                        debs = os.path.join(archives, '%s_*.deb' % conflict[0])
+                        for path in glob.glob(debs):
+                            ver = self._deb_version(path)
+                            if apt.apt_pkg.check_dep(ver, conflict[2],
+                                                          conflict[1]):
+                                os.unlink(path)
+
             if candidate.architecture != 'all':
                 if pkg + '-dbg' in c:
                     real_pkgs.add(pkg + '-dbg')
@@ -556,24 +646,26 @@ class __AptDpkgPackageInfo(PackageInfo):
                     real_pkgs.add(pkg + '-dbgsym')
                     if c[pkg + '-dbgsym'].candidate.version != candidate.version:
                         obsolete += 'outdated debug symbol package for %s: package version %s dbgsym version %s\n' % (
-                                pkg, candidate.version, c[pkg + '-dbgsym'].candidate.version)
+                            pkg, candidate.version, c[pkg + '-dbgsym'].candidate.version)
 
         for p in real_pkgs:
             c[p].mark_install(False, False)
 
+        last_written = time.time()
         # fetch packages
         fetcher = apt.apt_pkg.Acquire(fetchProgress)
         try:
             c.fetch_archives(fetcher=fetcher)
         except apt.cache.FetchFailedException as e:
             apport.error('Package download error, try again later: %s', str(e))
-            sys.exit(99) # transient error
+            sys.exit(99)  # transient error
 
         # unpack packages
         if verbose:
             print('Extracting downloaded debs...')
         for i in fetcher.items:
-            subprocess.check_call(['dpkg', '-x', i.destfile, rootdir])
+            if not permanent_rootdir or os.path.getctime(i.destfile) > last_written:
+                subprocess.check_call(['dpkg', '-x', i.destfile, rootdir])
             real_pkgs.remove(os.path.basename(i.destfile).split('_', 1)[0])
 
         if tmp_aptroot:
@@ -583,6 +675,8 @@ class __AptDpkgPackageInfo(PackageInfo):
         assert not real_pkgs, 'apt fetcher did not fetch these packages: ' \
             + ' '.join(real_pkgs)
 
+        if permanent_rootdir:
+            self._save_virtual_mapping(aptroot)
         return obsolete
 
     def package_name_glob(self, nameglob):
@@ -594,12 +688,13 @@ class __AptDpkgPackageInfo(PackageInfo):
     # Internal helper methods
     #
 
-    def _call_dpkg(self, args):
+    @classmethod
+    def _call_dpkg(klass, args):
         '''Call dpkg with given arguments and return output, or return None on
         error.'''
 
         dpkg = subprocess.Popen(['dpkg'] + args, stdout=subprocess.PIPE,
-            stderr=subprocess.PIPE)
+                                stderr=subprocess.PIPE)
         out = dpkg.communicate(input)[0].decode('UTF-8')
         if dpkg.returncode == 0:
             return out
@@ -610,18 +705,19 @@ class __AptDpkgPackageInfo(PackageInfo):
         '''Internal function for calling md5sum.
 
         This is separate from get_modified_files so that it is automatically
-        testable.'''
-
+        testable.
+        '''
         if os.path.exists(sumfile):
             m = subprocess.Popen(['/usr/bin/md5sum', '-c', sumfile],
-                stdout=subprocess.PIPE, stderr=subprocess.PIPE, close_fds=True,
-                cwd='/', env={})
-            out = m.communicate()[0].decode(errors='replace')
+                                 stdout=subprocess.PIPE, stderr=subprocess.PIPE,
+                                 cwd='/', env={})
+            out = m.communicate()[0].decode('UTF-8', errors='replace')
         else:
+            assert type(sumfile) == bytes, 'md5sum list value must be a byte array'
             m = subprocess.Popen(['/usr/bin/md5sum', '-c'],
-                stdin=subprocess.PIPE, stdout=subprocess.PIPE,
-                stderr=subprocess.PIPE, close_fds=True, cwd='/', env={})
-            out = m.communicate(sumfile)[0].decode(errors='replace')
+                                 stdin=subprocess.PIPE, stdout=subprocess.PIPE,
+                                 stderr=subprocess.PIPE, cwd='/', env={})
+            out = m.communicate(sumfile)[0].decode('UTF-8', errors='replace')
 
         # if md5sum succeeded, don't bother parsing the output
         if m.returncode == 0:
@@ -675,15 +771,11 @@ class __AptDpkgPackageInfo(PackageInfo):
         if age is None or age >= 86400:
             # determine distro release code name
             lsb_release = subprocess.Popen(['lsb_release', '-sc'],
-                stdout=subprocess.PIPE)
+                                           stdout=subprocess.PIPE)
             release_name = lsb_release.communicate()[0].decode('UTF-8').strip()
             assert lsb_release.returncode == 0
 
             url = '%s/dists/%s/Contents-%s.gz' % (self._get_mirror(), release_name, arch)
-            try:
-                from urllib.request import urlopen
-            except ImportError:
-                from urllib import urlopen
 
             src = urlopen(url)
             with open(map, 'wb') as f:
@@ -701,7 +793,7 @@ class __AptDpkgPackageInfo(PackageInfo):
         # zgrep is magnitudes faster than a 'gzip.open/split() loop'
         package = None
         zgrep = subprocess.Popen(['zgrep', '-m1', '^%s[[:space:]]' % file, map],
-            stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+                                 stdout=subprocess.PIPE, stderr=subprocess.PIPE)
         out = zgrep.communicate()[0].decode('UTF-8')
         # we do not check the return code, since zgrep -m1 often errors out
         # with 'stdout: broken pipe'
@@ -718,6 +810,8 @@ class __AptDpkgPackageInfo(PackageInfo):
             os.makedirs(os.path.join(apt_root, 'var', 'lib', 'apt', 'lists', 'partial'))
             os.makedirs(os.path.join(apt_root, 'var', 'cache', 'apt', 'archives', 'partial'))
             os.makedirs(os.path.join(apt_root, 'var', 'lib', 'dpkg'))
+            os.makedirs(os.path.join(apt_root, 'etc', 'apt', 'apt.conf.d'))
+            os.makedirs(os.path.join(apt_root, 'etc', 'apt', 'preferences.d'))
 
         # install apt sources
         list_d = os.path.join(apt_root, 'etc', 'apt', 'sources.list.d')
@@ -750,6 +844,16 @@ class __AptDpkgPackageInfo(PackageInfo):
         else:
             os.makedirs(trusted_d)
 
+    @classmethod
+    def _deb_version(klass, pkg):
+        '''Return the version of a .deb file'''
+
+        dpkg = subprocess.Popen(['dpkg-deb', '-f', pkg, 'Version'], stdout=subprocess.PIPE)
+        out = dpkg.communicate(input)[0].decode('UTF-8')
+        assert dpkg.returncode == 0
+        assert out
+        return out
+
     def compare_versions(self, ver1, ver2):
         '''Compare two package versions.
 
