@@ -1,29 +1,33 @@
 import unittest, gzip, imp, subprocess, tempfile, shutil, os, os.path, time
+import glob, urllib
+from apt import apt_pkg
 
 if os.environ.get('APPORT_TEST_LOCAL'):
     impl = imp.load_source('', 'backends/packaging-apt-dpkg.py').impl
 else:
     from apport.packaging_impl import impl
 
-def _has_default_route():
-    '''Return if there is a default route.
 
-    This is a reasonable indicator that online tests can be run.
+def _has_internet():
+    '''Return if there is sufficient network connection for the tests.
+
+    This checks if http://ddebs.ubuntu.com/ can be downloaded from, to check if
+    we can run the online tests.
     '''
     if os.environ.get('SKIP_ONLINE_TESTS'):
         return False
-    if _has_default_route.cache is None:
-        _has_default_route.cache = False
-        route = subprocess.Popen(['/sbin/route', '-n'],
-            stdout=subprocess.PIPE)
-        for l in route.stdout:
-            if l.decode('UTF-8').startswith('0.0.0.0 '):
-                _has_default_route.cache = True
-        route.wait()
+    if _has_internet.cache is None:
+        _has_internet.cache = False
+        try:
+            f = urllib.request.urlopen('http://ddebs.ubuntu.com/dbgsym-release-key.asc', timeout=30)
+            if f.readline().startswith(b'-----BEGIN PGP'):
+                _has_internet.cache = True
+        except (IOError, urllib.error.URLError):
+            pass
+    return _has_internet.cache
 
-    return _has_default_route.cache
+_has_internet.cache = None
 
-_has_default_route.cache = None
 
 class T(unittest.TestCase):
     def setUp(self):
@@ -55,10 +59,9 @@ class T(unittest.TestCase):
                 fd.write('More stuff')
             # use one relative and one absolute path in checksums file
             with open(sumfile, 'wb') as fd:
-                fd.write('''2e41290da2fa3f68bd3313174467e3b5  %s
-f6423dfbc4faf022e58b4d3f5ff71a70  %s
-deadbeef000001111110000011110000  /bin/\xc3\xa4
-''' % (f1[1:], f2))
+                fd.write(b'2e41290da2fa3f68bd3313174467e3b5  ' + f1[1:].encode() + b'\n')
+                fd.write(b'f6423dfbc4faf022e58b4d3f5ff71a70  ' + f2.encode() + b'\n')
+                fd.write(b'deadbeef000001111110000011110000  /bin/\xc3\xa4')
             self.assertEqual(impl._check_files_md5(sumfile), [], 'correct md5sums')
 
             with open(f1, 'w') as fd:
@@ -72,9 +75,9 @@ deadbeef000001111110000011110000  /bin/\xc3\xa4
             self.assertEqual(impl._check_files_md5(sumfile), [f2], 'file 2 wrong')
 
             # check using a direct md5 list as argument
-            with open(sumfile) as fd:
+            with open(sumfile, 'rb') as fd:
                 self.assertEqual(impl._check_files_md5(fd.read()),
-                    [f2], 'file 2 wrong')
+                                 [f2], 'file 2 wrong')
 
         finally:
             shutil.rmtree(td)
@@ -96,21 +99,21 @@ deadbeef000001111110000011110000  /bin/\xc3\xa4
         '''get_dependencies().'''
 
         # package with both Depends: and Pre-Depends:
-        d  = impl.get_dependencies('bash')
+        d = impl.get_dependencies('bash')
         self.assertTrue(len(d) > 2)
         self.assertTrue('libc6' in d)
         for dep in d:
             self.assertTrue(impl.get_version(dep))
 
         # Pre-Depends: only
-        d  = impl.get_dependencies('coreutils')
+        d = impl.get_dependencies('coreutils')
         self.assertTrue(len(d) >= 1)
         self.assertTrue('libc6' in d)
         for dep in d:
             self.assertTrue(impl.get_version(dep))
 
         # Depends: only
-        d  = impl.get_dependencies('libc6')
+        d = impl.get_dependencies('libc6')
         self.assertTrue(len(d) >= 1)
         for dep in d:
             self.assertTrue(impl.get_version(dep))
@@ -148,7 +151,7 @@ deadbeef000001111110000011110000  /bin/\xc3\xa4
         self.assertRaises(ValueError, impl.get_architecture, 'nonexisting')
         # just assume that bash uses the native architecture
         d = subprocess.Popen(['dpkg', '--print-architecture'],
-            stdout=subprocess.PIPE)
+                             stdout=subprocess.PIPE)
         system_arch = d.communicate()[0].decode().strip()
         assert d.returncode == 0
         self.assertEqual(impl.get_architecture('bash'), system_arch)
@@ -172,7 +175,7 @@ deadbeef000001111110000011110000  /bin/\xc3\xa4
 
         # determine distro release code name
         lsb_release = subprocess.Popen(['lsb_release', '-sc'],
-            stdout=subprocess.PIPE)
+                                       stdout=subprocess.PIPE)
         release_name = lsb_release.communicate()[0].decode('UTF-8').strip()
         assert lsb_release.returncode == 0
 
@@ -182,7 +185,7 @@ deadbeef000001111110000011110000  /bin/\xc3\xa4
             mapdir = os.path.join(basedir, 'dists', release_name)
             os.makedirs(mapdir)
             with gzip.open(os.path.join(mapdir, 'Contents-%s.gz' %
-                impl.get_system_architecture()), 'w') as f:
+                                        impl.get_system_architecture()), 'w') as f:
                 f.write(b'''
 foo header
 FILE                                                    LOCATION
@@ -221,7 +224,7 @@ bo/gu/s                                                 na/mypackage
             # outdated cache, must refresh the cache and hit the invalid
             # mirror
             now = int(time.time())
-            os.utime(os.path.join(cache_dir, cache_file), (now, now-90000))
+            os.utime(os.path.join(cache_dir, cache_file), (now, now - 90000))
 
             self.assertRaises(IOError, impl.get_file_package, '/bo/gu/s', True, cache_dir)
         finally:
@@ -232,7 +235,7 @@ bo/gu/s                                                 na/mypackage
 
         # pick first diversion we have
         p = subprocess.Popen('LC_ALL=C dpkg-divert --list | head -n 1',
-            shell=True, stdout=subprocess.PIPE)
+                             shell=True, stdout=subprocess.PIPE)
         out = p.communicate()[0].decode('UTF-8')
         assert p.returncode == 0
         assert out
@@ -318,34 +321,34 @@ bo/gu/s                                                 na/mypackage
         self.assertEqual(impl.package_name_glob('bash'), ['bash'])
         self.assertEqual(impl.package_name_glob('xzywef*'), [])
 
-    @unittest.skipUnless(_has_default_route(), 'online test')
+    @unittest.skipUnless(_has_internet(), 'online test')
     def test_install_packages_versioned(self):
         '''install_packages() with versions and with cache'''
 
         self._setup_foonux_config()
         impl.install_packages(self.rootdir, self.configdir, 'Foonux 1.2',
-                [('coreutils', '7.4-2ubuntu2'),
-                 ('libc6', '2.11.1-0ubuntu7'),
-                 ('tzdata', '2010i-1'),
-                ], False, self.cachedir)
+                              [('coreutils', '7.4-2ubuntu2'),
+                               ('libc6', '2.11.1-0ubuntu7'),
+                               ('tzdata', '2010i-1'),
+                              ], False, self.cachedir)
 
         self.assertTrue(os.path.exists(os.path.join(self.rootdir,
-            'usr/bin/stat')))
+                                                    'usr/bin/stat')))
         self.assertTrue(os.path.exists(os.path.join(self.rootdir,
-            'usr/lib/debug/usr/bin/stat')))
+                                                    'usr/lib/debug/usr/bin/stat')))
         self.assertTrue(os.path.exists(os.path.join(self.rootdir,
-            'usr/share/zoneinfo/zone.tab')))
+                                                    'usr/share/zoneinfo/zone.tab')))
         self.assertTrue(os.path.exists(os.path.join(self.rootdir,
-            'usr/share/doc/libc6/copyright')))
+                                                    'usr/share/doc/libc6/copyright')))
 
         # does not clobber config dir
         self.assertEqual(os.listdir(self.configdir), ['Foonux 1.2'])
         self.assertEqual(os.listdir(os.path.join(self.configdir, 'Foonux 1.2')),
-                ['sources.list'])
+                         ['sources.list'])
 
         # caches packages
         cache = os.listdir(os.path.join(self.cachedir, 'Foonux 1.2', 'apt',
-            'var', 'cache', 'apt', 'archives'))
+                                        'var', 'cache', 'apt', 'archives'))
         cache_names = [p.split('_')[0] for p in cache]
         self.assertTrue('coreutils' in cache_names)
         self.assertTrue('coreutils-dbgsym' in cache_names)
@@ -356,31 +359,31 @@ bo/gu/s                                                 na/mypackage
         # installs cached packages
         os.unlink(os.path.join(self.rootdir, 'usr/bin/stat'))
         impl.install_packages(self.rootdir, self.configdir, 'Foonux 1.2',
-                [('coreutils', '7.4-2ubuntu2'),
-                ], False, self.cachedir)
-        self.assertTrue(os.path.exists(os.path.join(self.rootdir,
-            'usr/bin/stat')))
+                              [('coreutils', '7.4-2ubuntu2'),
+                              ], False, self.cachedir)
+        self.assertTrue(os.path.exists(
+            os.path.join(self.rootdir, 'usr/bin/stat')))
 
         # complains about obsolete packages
         result = impl.install_packages(self.rootdir, self.configdir,
-                'Foonux 1.2', [('gnome-common', '1.1')])
+                                       'Foonux 1.2', [('gnome-common', '1.1')])
         self.assertEqual(len(result.splitlines()), 1)
         self.assertTrue('gnome-common' in result)
         self.assertTrue('1.1' in result)
         # ... but installs the current version anyway
-        self.assertTrue(os.path.exists(os.path.join(self.rootdir,
-            'usr/bin/gnome-autogen.sh')))
+        self.assertTrue(os.path.exists(
+            os.path.join(self.rootdir, 'usr/bin/gnome-autogen.sh')))
 
         # does not crash on nonexisting packages
         result = impl.install_packages(self.rootdir, self.configdir,
-                'Foonux 1.2', [('buggerbogger', None)])
+                                       'Foonux 1.2', [('buggerbogger', None)])
         self.assertEqual(len(result.splitlines()), 1)
         self.assertTrue('buggerbogger' in result)
         self.assertTrue('not exist' in result)
 
         # can interleave with other operations
         dpkg = subprocess.Popen(['dpkg-query', '-Wf${Version}', 'dash'],
-                stdout=subprocess.PIPE)
+                                stdout=subprocess.PIPE)
         coreutils_version = dpkg.communicate()[0].decode()
         self.assertEqual(dpkg.returncode, 0)
 
@@ -390,40 +393,40 @@ bo/gu/s                                                 na/mypackage
         # still installs packages after above operations
         os.unlink(os.path.join(self.rootdir, 'usr/bin/stat'))
         impl.install_packages(self.rootdir, self.configdir, 'Foonux 1.2',
-                [('coreutils', '7.4-2ubuntu2'),
-                 ('dpkg', None),
-                ], False, self.cachedir)
+                              [('coreutils', '7.4-2ubuntu2'),
+                               ('dpkg', None),
+                              ], False, self.cachedir)
         self.assertTrue(os.path.exists(os.path.join(self.rootdir,
-            'usr/bin/stat')))
+                                                    'usr/bin/stat')))
         self.assertTrue(os.path.exists(os.path.join(self.rootdir,
-            'usr/bin/dpkg')))
+                                                    'usr/bin/dpkg')))
 
-    @unittest.skipUnless(_has_default_route(), 'online test')
+    @unittest.skipUnless(_has_internet(), 'online test')
     def test_install_packages_unversioned(self):
         '''install_packages() without versions and no cache'''
 
         self._setup_foonux_config()
         impl.install_packages(self.rootdir, self.configdir, 'Foonux 1.2',
-                [('coreutils', None),
-                 ('tzdata', None),
-                ], False, None)
+                              [('coreutils', None),
+                               ('tzdata', None),
+                              ], False, None)
 
         self.assertTrue(os.path.exists(os.path.join(self.rootdir,
-            'usr/bin/stat')))
+                                                    'usr/bin/stat')))
         self.assertTrue(os.path.exists(os.path.join(self.rootdir,
-            'usr/lib/debug/usr/bin/stat')))
+                                                    'usr/lib/debug/usr/bin/stat')))
         self.assertTrue(os.path.exists(os.path.join(self.rootdir,
-            'usr/share/zoneinfo/zone.tab')))
+                                                    'usr/share/zoneinfo/zone.tab')))
 
         # does not clobber config dir
         self.assertEqual(os.listdir(self.configdir), ['Foonux 1.2'])
         self.assertEqual(os.listdir(os.path.join(self.configdir, 'Foonux 1.2')),
-                ['sources.list'])
+                         ['sources.list'])
 
         # no cache
         self.assertEqual(os.listdir(self.cachedir), [])
 
-    @unittest.skipUnless(_has_default_route(), 'online test')
+    @unittest.skipUnless(_has_internet(), 'online test')
     def test_install_packages_system(self):
         '''install_packages() with system configuration'''
 
@@ -434,14 +437,14 @@ bo/gu/s                                                 na/mypackage
 
         self._setup_foonux_config()
         result = impl.install_packages(self.rootdir, None, None,
-                [('coreutils', impl.get_version('coreutils')),
-                 ('tzdata', '1.1'),
-                ], False, self.cachedir)
+                                       [('coreutils', impl.get_version('coreutils')),
+                                        ('tzdata', '1.1'),
+                                       ], False, self.cachedir)
 
         self.assertTrue(os.path.exists(os.path.join(self.rootdir,
-            'usr/bin/stat')))
+                                                    'usr/bin/stat')))
         self.assertTrue(os.path.exists(os.path.join(self.rootdir,
-            'usr/share/zoneinfo/zone.tab')))
+                                                    'usr/share/zoneinfo/zone.tab')))
 
         # complains about obsolete packages
         self.assertEqual(len(result.splitlines()), 1)
@@ -450,7 +453,7 @@ bo/gu/s                                                 na/mypackage
 
         # caches packages
         cache = os.listdir(os.path.join(self.cachedir, 'system', 'apt',
-            'var', 'cache', 'apt', 'archives'))
+                                        'var', 'cache', 'apt', 'archives'))
         cache_names = [p.split('_')[0] for p in cache]
         self.assertTrue('coreutils' in cache_names)
         self.assertEqual('coreutils-dbgsym' in cache_names, self.has_dbgsym)
@@ -462,13 +465,13 @@ bo/gu/s                                                 na/mypackage
         try:
             os.chdir(self.workdir)
             impl.install_packages('root', None, None,
-                    [('coreutils', None)], False, 'cache')
+                                  [('coreutils', None)], False, 'cache')
         finally:
             os.chdir(orig_cwd)
-        self.assertTrue(os.path.exists(os.path.join(self.rootdir,
-            'usr/bin/stat')))
+            self.assertTrue(os.path.exists(os.path.join(self.rootdir,
+                                                        'usr/bin/stat')))
 
-    @unittest.skipUnless(_has_default_route(), 'online test')
+    @unittest.skipUnless(_has_internet(), 'online test')
     def test_install_packages_error(self):
         '''install_packages() with errors'''
 
@@ -479,7 +482,7 @@ bo/gu/s                                                 na/mypackage
 
         try:
             impl.install_packages(self.rootdir, self.configdir, 'Foonux 1.2',
-                    [('tzdata', None)], False, self.cachedir)
+                                  [('tzdata', None)], False, self.cachedir)
             self.fail('install_packages() unexpectedly succeeded with broken sources.list')
         except SystemError as e:
             self.assertTrue('bogus' in str(e))
@@ -491,12 +494,87 @@ bo/gu/s                                                 na/mypackage
 
         try:
             impl.install_packages(self.rootdir, self.configdir, 'Foonux 1.2',
-                    [('tzdata', None)], False, self.cachedir)
+                                  [('tzdata', None)], False, self.cachedir)
             self.fail('install_packages() unexpectedly succeeded with broken server URL')
         except SystemError as e:
             self.assertTrue('nosuchdistro' in str(e), str(e))
             self.assertTrue('index files failed to download' in str(e))
 
+    @unittest.skipUnless(_has_internet(), 'online test')
+    def test_install_packages_permanent_sandbox(self):
+        '''install_packages() with a permanent sandbox'''
+
+        self._setup_foonux_config()
+        zonetab = os.path.join(self.rootdir, 'usr/share/zoneinfo/zone.tab')
+
+        impl.install_packages(self.rootdir, self.configdir, 'Foonux 1.2',
+                              [('tzdata', None)], False, self.cachedir, permanent_rootdir=True)
+
+        # This will now be using a Cache with our rootdir.
+        archives = apt_pkg.config.find_dir('Dir::Cache::archives')
+        tzdata = glob.glob(os.path.join(archives, 'tzdata*.deb'))
+        if not tzdata:
+            self.fail('tzdata was not downloaded')
+        tzdata_written = os.path.getctime(tzdata[0])
+        zonetab_written = os.path.getctime(zonetab)
+
+        impl.install_packages(self.rootdir, self.configdir, 'Foonux 1.2',
+                              [('coreutils', None), ('tzdata', None)], False, self.cachedir,
+                              permanent_rootdir=True)
+
+        if not glob.glob(os.path.join(archives, 'coreutils*.deb')):
+            self.fail('coreutils was not downloaded.')
+            self.assertEqual(os.path.getctime(tzdata[0]), tzdata_written,
+                             'tzdata downloaded twice.')
+            self.assertEqual(zonetab_written, os.path.getctime(zonetab),
+                             'zonetab written twice.')
+            self.assertTrue(os.path.exists(
+                os.path.join(self.rootdir, 'usr/bin/stat')))
+
+        # Prevent packages from downloading.
+        apt_pkg.config.set('Acquire::http::Proxy', 'http://nonexistent')
+        self.assertRaises(SystemExit, impl.install_packages, self.rootdir,
+                          self.configdir, 'Foonux 1.2', [('libc6', None)], False,
+                          self.cachedir, permanent_rootdir=True)
+        # These packages exist, so attempting to install them should not fail.
+        impl.install_packages(self.rootdir, self.configdir, 'Foonux 1.2',
+                              [('coreutils', None), ('tzdata', None)], False, self.cachedir,
+                              permanent_rootdir=True)
+        apt_pkg.config.set('Acquire::http::Proxy', '')
+
+    @unittest.skipUnless(_has_internet(), 'online test')
+    def test_install_packages_permanent_sandbox_repack(self):
+        self._setup_foonux_config()
+        apache_bin_path = os.path.join(self.rootdir, 'usr/sbin/apache2')
+        impl.install_packages(self.rootdir, self.configdir, 'Foonux 1.2',
+                              [('apache2-mpm-worker', None)], False, self.cachedir,
+                              permanent_rootdir=True)
+        self.assertTrue(os.readlink(apache_bin_path).endswith('mpm-worker/apache2'))
+        impl.install_packages(self.rootdir, self.configdir, 'Foonux 1.2',
+                              [('apache2-mpm-event', None)], False, self.cachedir,
+                              permanent_rootdir=True)
+        self.assertTrue(os.readlink(apache_bin_path).endswith('mpm-event/apache2'),
+                        'should have installed mpm-event, but have mpm-worker.')
+        impl.install_packages(self.rootdir, self.configdir, 'Foonux 1.2',
+                              [('apache2-mpm-worker', None)], False, self.cachedir,
+                              permanent_rootdir=True)
+        self.assertTrue(os.readlink(apache_bin_path).endswith('mpm-worker/apache2'),
+                        'should have installed mpm-worker, but have mpm-event.')
+
+    @unittest.skipUnless(_has_internet(), 'online test')
+    def test_get_source_tree_sandbox(self):
+        self._setup_foonux_config()
+        out_dir = os.path.join(self.workdir, 'out')
+        os.mkdir(out_dir)
+        impl._build_apt_sandbox(self.rootdir, os.path.join(self.configdir, 'Foonux 1.2', 'sources.list'))
+        res = impl.get_source_tree('base-files', out_dir, sandbox=self.rootdir,
+                                   apt_update=True)
+        self.assertTrue(os.path.isdir(os.path.join(res, 'debian')))
+        # this needs to be updated when the release in _setup_foonux_config
+        # changes
+        self.assertTrue(res.endswith('/base-files-5.0.0ubuntu20'),
+                        'unexpected version: ' + res.split('/')[-1])
+
     def _setup_foonux_config(self):
         '''Set up directories and configuration for install_packages()'''
 
@@ -509,12 +587,13 @@ bo/gu/s                                                 na/mypackage
         os.mkdir(os.path.join(self.configdir, 'Foonux 1.2'))
         with open(os.path.join(self.configdir, 'Foonux 1.2', 'sources.list'), 'w') as f:
             f.write('deb http://archive.ubuntu.com/ubuntu/ lucid main\n')
+            f.write('deb-src http://archive.ubuntu.com/ubuntu/ lucid main\n')
             f.write('deb http://ddebs.ubuntu.com/ lucid main\n')
 
 # only execute if dpkg is available
 try:
     if subprocess.call(['dpkg', '--help'], stdout=subprocess.PIPE,
-        stderr=subprocess.PIPE) == 0:
+                       stderr=subprocess.PIPE) == 0:
         unittest.main()
 except OSError:
     pass
